==> grizzly-1.0.0.tar.gz: lib/src/double/double_mat_iterable.dart <==
import 'package:grizzly/grizzly.dart';
import 'package:grizzly/src/utils/comap.dart';

typedef Double2DView = Iterable<Iterable<double>>;

extension DoubleMatIterableExt on Iterable<Iterable<double>> {
  List<List<double>> matmult(List<List<num>> other) {
    final ret = <List<double>>[];
    for (final row in this) {
      final outRow = <double>[];
      for (final col in other.cols) {
        outRow.add(row.dot(col));
      }
      ret.add(outRow);
    }
    return ret;
  }

  /// Returns column vector resulting from the matrix multiplication of this * other.
  /// Considers [other] as a column matrix of form m x 1.
  Iterable<double> matmultColVector(Iterable<num> other) =>
      map((row) => row.dot(other));

  Iterable<Iterable<double>> plus(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row.plus(other));
    } else if (other is Iterable<num>) {
      return map((e) => e.plus(other));
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1.plus(row2));
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<double>> operator -(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row - other);
    } else if (other is Iterable<num>) {
      return map((e) => e - other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 - row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<double>> operator *(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row * other);
    } else if (other is Iterable<num>) {
      return map((e) => e * other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 * row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<double>> operator /(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row / other);
    } else if (other is Iterable<num>) {
      return map((e) => e / other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 / row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<int>> operator ~/(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row ~/ other);
    } else if (other is Iterable<num>) {
      return map((e) => e ~/ other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 ~/ row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  bool isClose(Iterable<Iterable<num>> v, {double tol = 1e-8}) {
    final iter1 = iterator;
    final iter2 = v.iterator;
    while (iter1.moveNext() && iter2.moveNext()) {
      if (!iter1.current.isClose(iter2.current)) {
        return false;
      }
    }
    return true;
  }

  bool isCloseVector(Iterable<num> v, {double tol = 1e-8}) {
    for (final row in this) {
      if (!row.isClose(v, tol: tol)) return false;
    }
    return true;
  }

  bool isCloseScalar(num v, {double tol = 1e-8}) {
    for (final row in this) {
      if (!row.isCloseScalar(v, tol: tol)) return false;
    }
    return true;
  }

  Iterable<double> dot([Iterable<Iterable<num>>? other]) => comapEq(
      this, other ?? this, (Iterable<double> a, Iterable<num> b) => a.dot(b));
}

==> grizzly-1.0.4.tar.gz: lib/src/double/double_mat_iterable.dart <==
import 'package:grizzly/grizzly.dart';
import 'package:grizzly/src/utils/comap.dart';
import 'package:grizzly/src/utils/exception.dart';

typedef Double2DView = Iterable<Iterable<double>>;

extension DoubleMatIterableExt on Iterable<Iterable<double>> {
  List<List<double>> matmult(List<List<num>> other) {
    final ret = <List<double>>[];
    for (final row in this) {
      final outRow = <double>[];
      for (final col in other.cols) {
        outRow.add(row.dot(col));
      }
      ret.add(outRow);
    }
    return ret;
  }

  /// Returns column vector resulting from the matrix multiplication of this * other.
  /// Considers [other] as a column matrix of form m x 1.
  Iterable<double> matmultColVector(Iterable<num> other) =>
      map((row) => row.dot(other));

  Iterable<Iterable<double>> plus(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row.plus(other));
    } else if (other is Iterable<num>) {
      return map((e) => e.plus(other));
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1.plus(row2));
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<double>> operator -(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row - other);
    } else if (other is Iterable<num>) {
      return map((e) => e - other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 - row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<double>> operator *(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row * other);
    } else if (other is Iterable<num>) {
      return map((e) => e * other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 * row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<double>> operator /(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row / other);
    } else if (other is Iterable<num>) {
      return map((e) => e / other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 / row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  Iterable<Iterable<int>> operator ~/(
      /* Iterable<Iterable<num>> | Iterable<num> | num */ other) {
    if (other is num) {
      return map((row) => row ~/ other);
    } else if (other is Iterable<num>) {
      return map((e) => e ~/ other);
    } else if (other is Iterable<Iterable<num>>) {
      return comapEq(this, other,
          (Iterable<double> row1, Iterable<num> row2) => row1 ~/ row2);
    }

    throw UnsupportedError(
        'only Iterable<Iterable<num>> | Iterable<num> | num is supported');
  }

  bool isEqual(Iterable<Iterable<num>> v, {double eps = 1e-8}) {
    final iter1 = iterator;
    final iter2 = v.iterator;
    while (iter1.moveNext() && iter2.moveNext()) {
      if (!iter1.current.isEqual(iter2.current, eps: eps)) {
        return false;
      }
    }
    return true;
  }

  bool assertEqual(Iterable<Iterable<num>> v, {double eps = 1e-8}) {
    final iter1 = iterator;
    final iter2 = v.iterator;
    int i = 0;
    while (iter1.moveNext() && iter2.moveNext()) {
      try {
        iter1.current.assertEqual(iter2.current);
      } on AssertionError catch (e) {
        throw AssertionError('$i:${e.message}');
      }
      i++;
    }
    return true;
  }

  bool isEqualVector(Iterable<num> v, {double eps = 1e-8}) {
    for (final row in this) {
      if (!row.isEqual(v, eps: eps)) return false;
    }
    return true;
  }

  bool isEqualScalar(num v, {double eps = 1e-8}) {
    for (final row in this) {
      if (!row.isEqualScalar(v, eps: eps)) return false;
    }
    return true;
  }

  Iterable<double> dot([Iterable<Iterable<num>>? other]) => comapEq(
      this, other ?? this, (Iterable<double> a, Iterable<num> b) => a.dot(b));
}
